@@ -22,6 +22,7 @@ objects.
 
 use Moo;
 use Carp 'croak';
+use Cwd 'abs_path';
 use YAML::XS;
 use File::Spec;
 use File::Stat;
@@ -49,7 +50,9 @@ has meta => (
         if (! -e $meta) {
             croak "No meta file found for the blog : $meta";
         }
-        return YAML::XS::LoadFile($meta);
+        my $yaml = YAML::XS::LoadFile($meta);
+        $self->info("Meta loaded from : '$meta'");
+        return $yaml;
     },
 );
 
@@ -83,8 +86,6 @@ has elements => (
     default => sub {
         my ($self) = @_;
         
-        #TODO: warn "in elements";
-
         my @posts;
         foreach my $path (@{ $self->meta->{featured_posts} }) {
             my $post;
@@ -94,18 +95,12 @@ has elements => (
                 )
             };
             if ($@) {
-                #TODO: warn "error [$path] : $@"; # invalid path for a LiteBlog::Article object
+                $self->error("Invalid path '$path' : $@"); 
                 next;
             }
-
-            eval { $post->content && $post->title };
-            if ($@) {
-                #TODO: warn "Error in content or title [$path] : $@"; # invalid LiteBlog::Article object: no content & meta
-                next;
-            }
-
-            #TODO: warn "success for post";
+            
             # At this point, we're sure the post is OK to be rendered.
+            $self->info("Post initialized : ".$post->title);
             push @posts, $post;
         }
         
@@ -167,12 +162,17 @@ sub select_articles {
     # Load Article objects up to the limit
     foreach my $dir (@dirs) {
         my $article;
-        $article = Dancer2::Plugin::LiteBlog::Article->new( 
-            basedir => File::Spec->catdir($root, $dir) );
-
+        eval { 
+            $article = Dancer2::Plugin::LiteBlog::Article->new( 
+                basedir => File::Spec->catdir($root, $dir),
+                base_path => $self->mount,
+            );
+        };
         # make sur this is a valid article 
-        eval { $article->content };
-        next if $@;
+        if ($@) {
+            $self->info("Not a valid article '$root/$dir' : $@, skipping");
+            next;
+        }
 
         push @records, $article;
         last if ++$count == $limit;
@@ -269,26 +269,50 @@ Examples:
 
 =cut
 
+
+# returns the right prefix, based on mount.
+sub _get_prefix {
+    my ($self, $mount) = @_;
+    $mount ||= '/blog'; # default value is /blog
+    
+    # handle top-level mount
+    return "" if $mount eq '/';
+    return $mount;
+}
+
 sub declare_routes {
     my ($self, $plugin, $config) = @_;
 
-    my $prefix = $config->{mount} || '/blog';
-    $plugin->dsl->info("LiteBlog::Blog - declaring route ${prefix}/:cat/:slug");
+    my $prefix = $self->_get_prefix($config->{mount});
+    $self->info("declaring route ${prefix}/:cat/:slug");
 
-    # /blog/:category/:permalink
+    # redirect missing trailing / 
     $plugin->app->add_route(
-        method  => 'get',
+        method => 'get',
         regexp  => "${prefix}/:cat/:slug",
         code    => sub {
             my $cat  = $plugin->dsl->param('cat');
             my $slug = $plugin->dsl->param('slug');
+            $self->info("In $prefix/$cat/$slug");
+
+            $plugin->dsl->redirect("$prefix/$cat/$slug/");
+        },
+    );
+
+    # /blog/:category/:permalink
+    $self->info("declaring route ${prefix}/:cat/:slug/");
+    $plugin->app->add_route(
+        method  => 'get',
+        regexp  => "${prefix}/:cat/:slug/",
+        code    => sub {
+            my $cat  = $plugin->dsl->param('cat');
+            my $slug = $plugin->dsl->param('slug');
+            $self->info("In $prefix/$cat/$slug");
 
-            $plugin->dsl->info("Looking for article: $cat/$slug");
             my $article = $self->find_article(category => $cat, path => $slug );
 
             if (! defined $article) {
-                $plugin->dsl->info("Article not found : $cat/$slug");
-                return $plugin->dsl->status('not_found');
+                return $plugin->render_client_error("Article not found : $cat/$slug");
             }
             # TODO hanlde invalid/missing $article->content as a 404
 
@@ -297,13 +321,14 @@ sub declare_routes {
                 {
                     page_title => $article->title,
                     content    => $article->content, 
+                    page_image => $article->image,
                     meta       => [
                         { 
-                            label => "In: ".$article->category, 
+                            label => $article->category, 
                             link => "$prefix/$cat" 
                         },
                         { 
-                            label => "Last update: ".$article->published_date 
+                            label => $article->published_date 
                         }
                     ],
                 },
@@ -313,6 +338,107 @@ sub declare_routes {
         }
     );
 
+=head3 GET C</blog/:category/:slug/:asset>
+
+If the C<:asset> is a readable file in the article's directory, 
+this route sends it back to the client. This is useful for hosting
+local files like images, PDF, etc in the article's folder.
+
+Example:
+
+    /blog/tech/some-article/featured.jpg
+
+=cut
+
+    $self->info("declaring route ${prefix}/:cat/:slug/:asset");
+    $plugin->app->add_route(
+        method => 'get',
+        regexp => "${prefix}/:category/:slug/:asset",
+        code   => sub {
+            my $cat = $plugin->dsl->param('category');
+            my $slug = $plugin->dsl->param('slug');
+            my $asset = $plugin->dsl->param('asset');
+            $self->info("in $prefix/$cat/$slug/$asset");
+
+            # the article must exist
+            my $article = $self->find_article(
+                category => $cat, path => $slug );
+            return $plugin->render_client_error(
+                "Requested article not found ($cat / $slug)") 
+                if ! defined $article;
+            $self->info("article is found ($cat/$slug)");
+            
+            # the asset file must exist in the article's basedir
+            my $asset_file = abs_path(File::Spec->catfile($article->basedir, $asset));
+            return $plugin->render_client_error(
+                "Asset file '$asset' does not exist"
+            ) if ! -e $asset_file;
+            $self->info("asset is found ($asset_file)");
+
+            return $plugin->dsl->send_file($asset_file, system_path => 1);
+        },
+    );
+
+=head3 GET C</:page>
+
+This is a catch-all route for retrieving and displaying any article based on its
+page path. If the article is not found, it will pass, this might be a category
+page.
+
+Examples:
+
+  /about-me
+  /contact
+
+=cut
+
+    # redirect to trailing / path
+    $self->info("declaring route ${prefix}/:page");
+    $plugin->app->add_route(
+        method => 'get',
+        regexp => "${prefix}/:page",
+        code => sub {
+            my $slug = $plugin->dsl->param('page');
+            $self->info("In $prefix/$slug");
+
+            $plugin->dsl->redirect("${prefix}/$slug/");
+        }
+    );
+
+    $self->info("declaring route ${prefix}/:page/");
+    $plugin->app->add_route(
+        method => 'get',
+        regexp => "${prefix}/:page/",
+        code => sub {
+            my $slug = $plugin->dsl->param('page');
+            $self->info("In $prefix/$slug/");
+            
+            my $article = $self->find_article(path => $slug );
+
+            if (! defined $article) {
+                $self->info("Not a page '$slug', passing");
+                return $plugin->dsl->pass();
+            }
+            $self->info("Got article : $article");
+            
+            # TODO hanlde invalid/missing $article->content as a 404
+            return $plugin->dsl->template(
+                'liteblog/single-page',
+                {
+                    page_title => $article->title,
+                    page_image => $article->image,
+                    content    => $article->content, 
+                    meta       => [
+                        { 
+                            label => "Last update: ".$article->published_date 
+                        }
+                    ],
+                },
+                { layout => 'liteblog'}
+            );
+        },
+    );
+
 =head3 GET C</blog/:category/>
 
 Displays a landing page for a specific category. If the category is not found or
@@ -326,19 +452,33 @@ Examples:
 
 =cut
 
-    # the /category landing page
+    # redirect missing trailing / 
+    $self->info("declaring route ${prefix}/:category");
     $plugin->app->add_route(
         method => 'get',
-        regexp => "${prefix}/:category/?",
+        regexp  => "${prefix}/:cat",
+        code    => sub {
+            my $cat  = $plugin->dsl->param('cat');
+            $self->info("In $prefix/$cat");
+
+            $plugin->dsl->redirect("$prefix/$cat/");
+        },
+    );
+
+    # the /category landing page
+    $self->info("declaring route ${prefix}/:category/");
+    $plugin->app->add_route(
+        method => 'get',
+        regexp => "${prefix}/:category/",
         code   => sub {
-            $plugin->dsl->info("in /$prefix/category route");
             my $category = $plugin->dsl->param('category');
+            $self->info("In $prefix/$category");
+
             if (! -d File::Spec->catdir($self->root, $category)) {
-                $plugin->dsl->info("Invalid category requested: '$category'");
-                return $plugin->dsl->status('not_found');
+                return $plugin->render_client_error("Invalid category requested: '$category'");
             }
             my $articles = $self->select_articles(category => $category, limit => 6);
-            $plugin->dsl->info("retrieved ".scalar(@$articles)." articles");
+            $self->info("retrieved ".scalar(@$articles)." articles");
             return $plugin->dsl->template(
                 'liteblog/single-page', {
                     page_title => ucfirst($category)." Stories",
@@ -353,48 +493,6 @@ Examples:
             );
         }
     );
-
-=head3 GET C</:page>
-
-This is a catch-all route for retrieving and displaying any article based on its
-page path. If the article is not found, it will return a 404 status. The
-rendering is done with the 'liteblog/single-page' template.
-
-Examples:
-
-  /about-me
-  /contact
-
-=cut
-
-    $plugin->app->add_route(
-        method => 'get',
-        regexp => '/:page',
-        code => sub {
-            my $slug = $plugin->dsl->param('page');
-            my $article = $self->find_article(path => $slug );
-
-            if (! defined $article) {
-                $plugin->dsl->info("Page not found : $slug");
-                return $plugin->dsl->status('not_found');
-            }
-            
-            # TODO hanlde invalid/missing $article->content as a 404
-            return $plugin->dsl->template(
-                'liteblog/single-page',
-                {
-                    page_title => $article->title,
-                    content    => $article->content, 
-                    meta       => [
-                        { 
-                            label => "Last update: ".$article->published_date 
-                        }
-                    ],
-                },
-                { layout => 'liteblog'}
-            );
-        },
-    );
 }
 
 # Private subs
